@@ -701,6 +701,7 @@ int SyntheticClient::run()
 	}
 	did_run_me();
       }
+      break;
 
     case SYNCLIENT_MODE_WRSHARED:
       {
@@ -1031,7 +1032,7 @@ int SyntheticClient::play_trace(Trace& t, string& prefix, bool metadata_only)
       ll_inos[1] = attr.st_ino;
       dout(5) << "'root' ino is " << inodeno_t(attr.st_ino) << dendl;
     } else {
-      dout(0) << "warning: play_trace coudln't lookup up my per-client directory" << dendl;
+      dout(0) << "warning: play_trace couldn't lookup up my per-client directory" << dendl;
     }
   }
 
@@ -2412,6 +2413,11 @@ int SyntheticClient::read_random(string& fn, int size, int rdsize)   // size is
 
     //dout(0) << "RANDOM NUMBER RETURN |" << x << "|" << dendl;
 
+    // cleanup before call 'new'
+    if (buf != NULL) {
+	delete[] buf;
+	buf = NULL;
+    }
     if ( x < 0.5) 
     {
         //dout(0) << "DECIDED TO READ " << x << dendl;
@@ -2580,6 +2586,11 @@ int SyntheticClient::read_random_ex(string& fn, int size, int rdsize)   // size
     
     //dout(0) << "RANDOM NUMBER RETURN |" << x << "|" << dendl;
     
+    // cleanup before call 'new'
+    if (buf != NULL) {
+	delete[] buf;
+	buf = NULL;
+    }
     if ( x < 0.5) 
       {
         //dout(0) << "DECIDED TO READ " << x << dendl;
@@ -3379,6 +3390,7 @@ int SyntheticClient::lookup_ino(inodeno_t ino)
 int SyntheticClient::chunk_file(string &filename)
 {
   int fd = client->open(filename.c_str(), O_RDONLY);
+  int ret;
 
   struct stat st;
   client->fstat(fd, &st);
@@ -3390,7 +3402,9 @@ int SyntheticClient::chunk_file(string &filename)
   inode_t inode;
   memset(&inode, 0, sizeof(inode));
   inode.ino = st.st_ino;
-  client->describe_layout(fd, &inode.layout);
+  ret = client->describe_layout(fd, &inode.layout);
+  if (ret < 0)
+    return ret;
 
   uint64_t pos = 0;
   bufferlist from_before;
