@@ -76,18 +76,11 @@ public:
     return r;
   }
 
-  int signal_exit(int r) {
-    if (forked) {
-      // tell parent.  this shouldn't fail, but if it does, pass the
-      // error back to the parent.
-      int ret = safe_write(fd[1], &r, sizeof(r));
-      if (ret <= 0)
-	return ret;
-    }
-    return r;
-  }
   void exit(int r) {
-    signal_exit(r);
+    if (forked) {
+      // tell parent
+      (void)::write(fd[1], &r, sizeof(r));
+    }
     ::exit(r);
   }
 
