@@ -18,12 +18,18 @@
 #include <iostream>
 #include <stdint.h>
 
+#include "include/buffer.h"
+#include "include/atomic.h"
+
 class AdminSocket;
 class CephContextServiceThread;
 class PerfCountersCollection;
 class md_config_obs_t;
-class md_config_t;
-class PerfCountersHook;
+struct md_config_t;
+class CephContextHook;
+class CryptoNone;
+class CryptoAES;
+class CryptoHandler;
 
 namespace ceph {
   class HeartbeatMap;
@@ -32,6 +38,8 @@ namespace ceph {
   }
 }
 
+using ceph::bufferlist;
+
 /* A CephContext represents the context held by a single library user.
  * There can be multiple CephContexts in the same process.
  *
@@ -42,9 +50,22 @@ namespace ceph {
 class CephContext {
 public:
   CephContext(uint32_t module_type_);
-  ~CephContext();
-  md_config_t *_conf;
 
+  // ref count!
+private:
+  ~CephContext();
+  atomic_t nref;
+public:
+  CephContext *get() {
+    nref.inc();
+    return this;
+  }
+  void put() {
+    if (nref.dec() == 0)
+      delete this;
+  }
+
+  md_config_t *_conf;
   ceph::log::Log *_log;
 
   /* Start the Ceph Context's service thread */
@@ -73,6 +94,16 @@ public:
    */
   AdminSocket *get_admin_socket();
 
+  /**
+   * process an admin socket command
+   */
+  void do_command(std::string command, std::string args, bufferlist *out);
+
+  /**
+   * get a crypto handler
+   */
+  CryptoHandler *get_crypto_handler(int type);
+
 private:
   CephContext(const CephContext &rhs);
   CephContext &operator=(const CephContext &rhs);
@@ -100,9 +131,13 @@ private:
 
   md_config_obs_t *_perf_counters_conf_obs;
 
-  PerfCountersHook *_perf_counters_hook;
+  CephContextHook *_admin_hook;
 
   ceph::HeartbeatMap *_heartbeat_map;
+
+  // crypto
+  CryptoNone *_crypto_none;
+  CryptoAES *_crypto_aes;
 };
 
 #endif
