@@ -6,6 +6,7 @@
 #include "common/ceph_context.h"
 #include "common/dout.h"
 #include "common/errno.h"
+#include "common/Throttle.h"
 #include "cls/lock/cls_lock_client.h"
 #include "include/inttypes.h"
 #include "include/stringify.h"
@@ -20,6 +21,8 @@
 #include "librbd/parent_types.h"
 #include "include/util.h"
 
+#include "librados/snap_set_diff.h"
+
 #define dout_subsys ceph_subsys_rbd
 #undef dout_prefix
 #define dout_prefix *_dout << "librbd: "
@@ -149,22 +152,29 @@ namespace librbd {
     uint64_t period = ictx->get_stripe_period();
     uint64_t num_period = ((newsize + period - 1) / period);
     uint64_t delete_off = MIN(num_period * period, size);
-    uint64_t delete_start = num_period * ictx->get_stripe_count();     // first object we can delete free and clear
+    // first object we can delete free and clear
+    uint64_t delete_start = num_period * ictx->get_stripe_count();
     uint64_t num_objects = ictx->get_num_objects();
     uint64_t object_size = ictx->get_object_size();
 
     ldout(cct, 10) << "trim_image " << size << " -> " << newsize
 		   << " periods " << num_period
 		   << " discard to offset " << delete_off
-		   << " delete objects " << delete_start << " to " << (num_objects-1)
+		   << " delete objects " << delete_start
+		   << " to " << (num_objects-1)
 		   << dendl;
 
+    SimpleThrottle throttle(cct->_conf->rbd_concurrent_management_ops, true);
     if (delete_start < num_objects) {
       ldout(cct, 2) << "trim_image objects " << delete_start << " to "
 		    << (num_objects - 1) << dendl;
       for (uint64_t i = delete_start; i < num_objects; ++i) {
 	string oid = ictx->get_object_name(i);
-	ictx->data_ctx.remove(oid);
+	Context *req_comp = new C_SimpleThrottle(&throttle);
+	librados::AioCompletion *rados_completion =
+	  librados::Rados::aio_create_completion(req_comp, NULL, rados_ctx_cb);
+	ictx->data_ctx.aio_remove(oid, rados_completion);
+	rados_completion->release();
 	prog_ctx.update_progress((i - delete_start) * object_size,
 				 (num_objects - delete_start) * object_size);
       }
@@ -173,19 +183,30 @@ namespace librbd {
     // discard the weird boundary, if any
     if (delete_off > newsize) {
       vector<ObjectExtent> extents;
-      Striper::file_to_extents(ictx->cct, ictx->format_string, &ictx->layout, newsize, delete_off - newsize, extents);
+      Striper::file_to_extents(ictx->cct, ictx->format_string, &ictx->layout,
+			       newsize, delete_off - newsize, extents);
 
-      for (vector<ObjectExtent>::iterator p = extents.begin(); p != extents.end(); ++p) {
+      for (vector<ObjectExtent>::iterator p = extents.begin();
+	   p != extents.end(); ++p) {
 	ldout(ictx->cct, 20) << " ex " << *p << dendl;
+	Context *req_comp = new C_SimpleThrottle(&throttle);
+	librados::AioCompletion *rados_completion =
+	  librados::Rados::aio_create_completion(req_comp, NULL, rados_ctx_cb);
 	if (p->offset == 0) {
-	  ictx->data_ctx.remove(p->oid.name);
+	  ictx->data_ctx.aio_remove(p->oid.name, rados_completion);
 	} else {
 	  librados::ObjectWriteOperation op;
 	  op.truncate(p->offset);
-	  ictx->data_ctx.operate(p->oid.name, &op);
+	  ictx->data_ctx.aio_operate(p->oid.name, rados_completion, &op);
 	}
+	rados_completion->release();
       }
     }
+    int r = throttle.wait_for_ret();
+    if (r < 0) {
+      lderr(cct) << "warning: failed to remove some object(s): "
+		 << cpp_strerror(r) << dendl;
+    }
   }
 
   int read_rbd_info(IoCtx& io_ctx, const string& info_oid,
@@ -310,16 +331,28 @@ namespace librbd {
   {
     uint64_t numseg = ictx->get_num_objects();
     uint64_t bsize = ictx->get_object_size();
+    int r;
+    CephContext *cct = ictx->cct;
+    SimpleThrottle throttle(cct->_conf->rbd_concurrent_management_ops, true);
 
     for (uint64_t i = 0; i < numseg; i++) {
-      int r;
       string oid = ictx->get_object_name(i);
-      r = ictx->data_ctx.selfmanaged_snap_rollback(oid, snap_id);
-      ldout(ictx->cct, 10) << "selfmanaged_snap_rollback on " << oid << " to "
-			   << snap_id << " returned " << r << dendl;
+      Context *req_comp = new C_SimpleThrottle(&throttle);
+      librados::AioCompletion *rados_completion =
+	librados::Rados::aio_create_completion(req_comp, NULL, rados_ctx_cb);
+      librados::ObjectWriteOperation op;
+      op.selfmanaged_snap_rollback(snap_id);
+      ictx->data_ctx.aio_operate(oid, rados_completion, &op);
+      ldout(cct, 10) << "scheduling selfmanaged_snap_rollback on "
+		     << oid << " to " << snap_id << dendl;
       prog_ctx.update_progress(i * bsize, numseg * bsize);
-      if (r < 0 && r != -ENOENT)
-	return r;
+    }
+
+    r = throttle.wait_for_ret();
+    if (r < 0) {
+      ldout(cct, 10) << "failed to rollback at least one object: "
+		     << cpp_strerror(r) << dendl;
+      return r;
     }
     return 0;
   }
@@ -1460,6 +1493,19 @@ reprotect_and_return_err:
     return 0;
   }
 
+  bool snap_exists(ImageCtx *ictx, const char *snap_name)
+  {
+    ldout(ictx->cct, 20) << "snap_exists " << ictx << " " << snap_name << dendl;
+
+    int r = ictx_check(ictx);
+    if (r < 0)
+      return r;
+
+    RWLock::RLocker l(ictx->snap_lock);
+    return ictx->snaps_by_name.count(snap_name);
+  }
+
+
   int add_snap(ImageCtx *ictx, const char *snap_name)
   {
     uint64_t snap_id;
@@ -1837,10 +1883,54 @@ reprotect_and_return_err:
     return r;
   }
 
+  class C_CopyWrite : public Context {
+  public:
+    C_CopyWrite(SimpleThrottle *throttle, bufferlist *bl)
+      : m_throttle(throttle), m_bl(bl) {}
+    virtual void finish(int r) {
+      delete m_bl;
+      m_throttle->end_op(r);
+    }
+  private:
+    SimpleThrottle *m_throttle;
+    bufferlist *m_bl;
+  };
+
+  class C_CopyRead : public Context {
+  public:
+    C_CopyRead(SimpleThrottle *throttle, ImageCtx *dest, uint64_t offset,
+	       bufferlist *bl)
+      : m_throttle(throttle), m_dest(dest), m_offset(offset), m_bl(bl) {
+      m_throttle->start_op();
+    }
+    virtual void finish(int r) {
+      if (r < 0) {
+	lderr(m_dest->cct) << "error reading from source image at offset "
+			   << m_offset << ": " << cpp_strerror(r) << dendl;
+	delete m_bl;
+	m_throttle->end_op(r);
+	return;
+      }
+      assert(m_bl->length() == (size_t)r);
+      Context *ctx = new C_CopyWrite(m_throttle, m_bl);
+      AioCompletion *comp = aio_create_completion_internal(ctx, rbd_ctx_cb);
+      r = aio_write(m_dest, m_offset, m_bl->length(), m_bl->c_str(), comp);
+      if (r < 0) {
+	ctx->complete(r);
+	comp->release();
+	lderr(m_dest->cct) << "error writing to destination image at offset "
+			   << m_offset << ": " << cpp_strerror(r) << dendl;
+      }
+    }
+  private:
+    SimpleThrottle *m_throttle;
+    ImageCtx *m_dest;
+    uint64_t m_offset;
+    bufferlist *m_bl;
+  };
+
   int copy(ImageCtx *src, ImageCtx *dest, ProgressContext &prog_ctx)
   {
-    CopyProgressCtx cp(prog_ctx);
-
     src->md_lock.get_read();
     src->snap_lock.get_read();
     uint64_t src_size = src->get_image_size(src->snap_id);
@@ -1853,21 +1943,36 @@ reprotect_and_return_err:
     dest->snap_lock.put_read();
     dest->md_lock.put_read();
 
+    CephContext *cct = src->cct;
     if (dest_size < src_size) {
-      lderr(src->cct) << " src size " << src_size << " != dest size " << dest_size << dendl;
+      lderr(cct) << " src size " << src_size << " >= dest size "
+		 << dest_size << dendl;
       return -EINVAL;
     }
-
-    cp.destictx = dest;
-    cp.src_size = src_size;
-
-    int64_t r = read_iterate(src, 0, src_size, do_copy_extent, &cp);
-
-    if (r >= 0) {
-      // don't return total bytes read, which may not fit in an int
-      r = 0;
-      prog_ctx.update_progress(cp.src_size, cp.src_size);
+    int r;
+    SimpleThrottle throttle(cct->_conf->rbd_concurrent_management_ops, false);
+    uint64_t period = src->get_stripe_period();
+    for (uint64_t offset = 0; offset < src_size; offset += period) {
+      uint64_t len = min(period, src_size - offset);
+      bufferlist *bl = new bufferlist();
+      Context *ctx = new C_CopyRead(&throttle, dest, offset, bl);
+      AioCompletion *comp = aio_create_completion_internal(ctx, rbd_ctx_cb);
+      r = aio_read(src, offset, len, NULL, bl, comp);
+      if (r < 0) {
+	ctx->complete(r);
+	comp->release();
+	throttle.wait_for_ret();
+	lderr(cct) << "could not read from source image from "
+		   << offset << " to " << offset + len << ": "
+		   << cpp_strerror(r) << dendl;
+	return r;
+      }
+      prog_ctx.update_progress(offset, src_size);
     }
+
+    r = throttle.wait_for_ret();
+    if (r >= 0)
+      prog_ctx.update_progress(src_size, src_size);
     return r;
   }
 
@@ -1965,7 +2070,8 @@ reprotect_and_return_err:
   // 'flatten' child image by copying all parent's blocks
   int flatten(ImageCtx *ictx, ProgressContext &prog_ctx)
   {
-    ldout(ictx->cct, 20) << "flatten" << dendl;
+    CephContext *cct = ictx->cct;
+    ldout(cct, 20) << "flatten" << dendl;
 
     if (ictx->read_only)
       return -EROFS;
@@ -1973,7 +2079,7 @@ reprotect_and_return_err:
     int r;
     // ictx_check also updates parent data
     if ((r = ictx_check(ictx)) < 0) {
-      lderr(ictx->cct) << "ictx_check failed" << dendl;
+      lderr(cct) << "ictx_check failed" << dendl;
       return r;
     }
 
@@ -1982,6 +2088,7 @@ reprotect_and_return_err:
     uint64_t overlap;
     uint64_t overlap_periods;
     uint64_t overlap_objects;
+    ::SnapContext snapc;
 
     {
       RWLock::RLocker l(ictx->md_lock);
@@ -1990,14 +2097,15 @@ reprotect_and_return_err:
 
       // can't flatten a non-clone
       if (ictx->parent_md.spec.pool_id == -1) {
-	lderr(ictx->cct) << "image has no parent" << dendl;
+	lderr(cct) << "image has no parent" << dendl;
 	return -EINVAL;
       }
       if (ictx->snap_id != CEPH_NOSNAP || ictx->read_only) {
-	lderr(ictx->cct) << "snapshots cannot be flattened" << dendl;
+	lderr(cct) << "snapshots cannot be flattened" << dendl;
 	return -EROFS;
       }
 
+      snapc = ictx->snapc;
       assert(ictx->parent != NULL);
       assert(ictx->parent_md.overlap <= ictx->size);
 
@@ -2008,48 +2116,52 @@ reprotect_and_return_err:
       overlap_objects = overlap_periods * ictx->get_stripe_count();
     }
 
-    char *buf = new char[object_size];
+    SimpleThrottle throttle(cct->_conf->rbd_concurrent_management_ops, false);
 
     for (uint64_t ono = 0; ono < overlap_objects; ono++) {
-      prog_ctx.update_progress(ono, overlap_objects);
+      {
+	RWLock::RLocker l(ictx->parent_lock);
+	// stop early if the parent went away - it just means
+	// another flatten finished first, so this one is useless.
+	if (!ictx->parent) {
+	  r = 0;
+	  goto err;
+	}
+      }
 
       // map child object onto the parent
       vector<pair<uint64_t,uint64_t> > objectx;
-      Striper::extent_to_file(ictx->cct, &ictx->layout,
+      Striper::extent_to_file(cct, &ictx->layout,
 			    ono, 0, object_size,
 			    objectx);
       uint64_t object_overlap = ictx->prune_parent_extents(objectx, overlap);
       assert(object_overlap <= object_size);
 
-      RWLock::RLocker l(ictx->parent_lock);
-      // stop early if the parent went away - it just means
-      // another flatten finished first, so this one is useless.
-      if (!ictx->parent) {
-	r = 0;
-	goto err;
-      }
-      if ((r = read(ictx->parent, objectx, buf, NULL)) < 0) {
-	lderr(ictx->cct) << "reading from parent failed" << dendl;
+      bufferlist bl;
+      string oid = ictx->get_object_name(ono);
+      Context *comp = new C_SimpleThrottle(&throttle);
+      AioWrite *req = new AioWrite(ictx, oid, ono, 0, objectx, object_overlap,
+				   bl, snapc, CEPH_NOSNAP, comp);
+      r = req->send();
+      if (r < 0) {
+	lderr(cct) << "failed to flatten object " << oid << dendl;
 	goto err;
       }
 
-      // for actual amount read, if data is all zero, don't bother with block
-      if (!buf_is_zero(buf, r)) {
-	bufferlist bl;
-	string oid = ictx->get_object_name(ono);
-	bl.append(buf, r);
-	r = cls_client::copyup(&ictx->data_ctx, oid, bl);
-	if (r < 0) {
-	  lderr(ictx->cct) << "failed to copy block to child" << dendl;
-	  goto err;
-	}
-      }
+      prog_ctx.update_progress(ono, overlap_objects);
+    }
+
+    r = throttle.wait_for_ret();
+    if (r < 0) {
+      lderr(cct) << "failed to flatten at least one object: "
+		 << cpp_strerror(r) << dendl;
+      goto err;
     }
 
     // remove parent from this (base) image
     r = cls_client::remove_parent(&ictx->md_ctx, ictx->header_oid);
     if (r < 0) {
-      lderr(ictx->cct) << "error removing parent" << dendl;
+      lderr(cct) << "error removing parent" << dendl;
       return r;
     }
 
@@ -2058,11 +2170,11 @@ reprotect_and_return_err:
     // will be removed when the last snap goes away)
     ictx->snap_lock.get_read();
     if (ictx->snaps.empty()) {
-      ldout(ictx->cct, 2) << "removing child from children list..." << dendl;
+      ldout(cct, 2) << "removing child from children list..." << dendl;
       int r = cls_client::remove_child(&ictx->md_ctx, RBD_CHILDREN,
 				       ictx->parent_md.spec, ictx->id);
       if (r < 0) {
-	lderr(ictx->cct) << "error removing child from children list" << dendl;
+	lderr(cct) << "error removing child from children list" << dendl;
 	ictx->snap_lock.put_read();
 	return r;
       }
@@ -2070,10 +2182,12 @@ reprotect_and_return_err:
     ictx->snap_lock.put_read();
     notify_change(ictx->md_ctx, ictx->header_oid, NULL, ictx);
 
-    ldout(ictx->cct, 20) << "finished flattening" << dendl;
+    ldout(cct, 20) << "finished flattening" << dendl;
+
+    return 0;
 
   err:
-    delete[] buf;
+    throttle.wait_for_ret();
     return r;
   }
 
@@ -2184,9 +2298,10 @@ reprotect_and_return_err:
     Context *ctx = reinterpret_cast<Context *>(arg);
     AioCompletion *comp = reinterpret_cast<AioCompletion *>(cb);
     ctx->complete(comp->get_return_value());
+    comp->release();
   }
 
-  int64_t read_iterate(ImageCtx *ictx, uint64_t off, size_t len,
+  int64_t read_iterate(ImageCtx *ictx, uint64_t off, uint64_t len,
 		       int (*cb)(uint64_t, size_t, const char *, void *),
 		       void *arg)
   {
@@ -2234,7 +2349,6 @@ reprotect_and_return_err:
 	cond.Wait(mylock);
       mylock.Unlock();
 
-      c->release();
       if (ret < 0)
 	return ret;
 
@@ -2254,6 +2368,183 @@ reprotect_and_return_err:
     return total_read;
   }
 
+  int simple_diff_cb(uint64_t off, size_t len, int exists, void *arg)
+  {
+    // This reads the existing extents in a parent from the beginning
+    // of time.  Since images are thin-provisioned, the extents will
+    // always represent data, not holes.
+    assert(exists);
+    interval_set<uint64_t> *diff = static_cast<interval_set<uint64_t> *>(arg);
+    diff->insert(off, len);
+    return 0;
+  }
+
+
+  int diff_iterate(ImageCtx *ictx, const char *fromsnapname,
+		   uint64_t off, uint64_t len,
+		   int (*cb)(uint64_t, size_t, int, void *),
+		   void *arg)
+  {
+    utime_t start_time, elapsed;
+
+    ldout(ictx->cct, 20) << "diff_iterate " << ictx << " off = " << off
+			 << " len = " << len << dendl;
+
+    // ensure previous writes are visible to listsnaps
+    _flush(ictx);
+
+    int r = ictx_check(ictx);
+    if (r < 0)
+      return r;
+
+    r = clip_io(ictx, off, &len);
+    if (r < 0)
+      return r;
+
+    librados::IoCtx head_ctx;
+
+    ictx->md_lock.get_read();
+    ictx->snap_lock.get_read();
+    head_ctx.dup(ictx->data_ctx);
+    snap_t from_snap_id = 0;
+    uint64_t from_size = 0;
+    if (fromsnapname) {
+      from_snap_id = ictx->get_snap_id(fromsnapname);
+      from_size = ictx->get_image_size(from_snap_id);
+    }
+    snap_t end_snap_id = ictx->snap_id;
+    uint64_t end_size = ictx->get_image_size(end_snap_id);
+    ictx->snap_lock.put_read();
+    ictx->md_lock.put_read();
+    if (from_snap_id == CEPH_NOSNAP) {
+      return -ENOENT;
+    }
+    if (from_snap_id == end_snap_id) {
+      // no diff.
+      return 0;
+    }
+    if (from_snap_id >= end_snap_id) {
+      return -EINVAL;
+    }
+
+    // we must list snaps via the head, not end snap
+    head_ctx.snap_set_read(CEPH_SNAPDIR);
+
+    ldout(ictx->cct, 20) << "diff_iterate from " << from_snap_id << " to " << end_snap_id
+			 << " size from " << from_size << " to " << end_size << dendl;
+
+    // FIXME: if end_size > from_size, we could read_iterate for the
+    // final part, and skip the listsnaps op.
+
+    // check parent overlap only if we are comparing to the beginning of time
+    interval_set<uint64_t> parent_diff;
+    if (from_snap_id == 0) {
+      ictx->parent_lock.get_read();
+      uint64_t overlap = end_size;
+      ictx->get_parent_overlap(from_snap_id, &overlap);
+      r = 0;
+      if (ictx->parent && overlap > 0) {
+	ldout(ictx->cct, 10) << " first getting parent diff" << dendl;
+	r = diff_iterate(ictx->parent, NULL, 0, overlap, simple_diff_cb, &parent_diff);
+      }
+      ictx->parent_lock.put_read();
+      if (r < 0)
+	return r;
+    }
+
+    uint64_t period = ictx->get_stripe_period();
+    uint64_t left = len;
+
+    while (left > 0) {
+      uint64_t period_off = off - (off % period);
+      uint64_t read_len = min(period_off + period - off, left);
+
+      // map to extents
+      map<object_t,vector<ObjectExtent> > object_extents;
+      Striper::file_to_extents(ictx->cct, ictx->format_string, &ictx->layout,
+			       off, read_len, object_extents, 0);
+
+      // get snap info for each object
+      for (map<object_t,vector<ObjectExtent> >::iterator p = object_extents.begin();
+	   p != object_extents.end();
+	   ++p) {
+	ldout(ictx->cct, 20) << "diff_iterate object " << p->first << dendl;
+
+	librados::snap_set_t snap_set;
+	int r = head_ctx.list_snaps(p->first.name, &snap_set);
+	if (r == -ENOENT) {
+	  if (from_snap_id == 0 && !parent_diff.empty()) {
+	    // report parent diff instead
+	    for (vector<ObjectExtent>::iterator q = p->second.begin(); q != p->second.end(); ++q) {
+	      for (vector<pair<uint64_t,uint64_t> >::iterator r = q->buffer_extents.begin();
+		   r != q->buffer_extents.end();
+		   ++r) {
+		interval_set<uint64_t> o;
+		o.insert(off + r->first, r->second);
+		o.intersection_of(parent_diff);
+		ldout(ictx->cct, 20) << " reporting parent overlap " << o << dendl;
+		for (interval_set<uint64_t>::iterator s = o.begin(); s != o.end(); ++s) {
+		  cb(s.get_start(), s.get_len(), true, arg);
+		}
+	      }
+	    }
+	  }
+	  continue;
+	}
+	if (r < 0)
+	  return r;
+
+	// calc diff from from_snap_id -> to_snap_id
+	interval_set<uint64_t> diff;
+	bool end_exists;
+	calc_snap_set_diff(ictx->cct, snap_set,
+			   from_snap_id,
+			   end_snap_id,
+			   &diff, &end_exists);
+	ldout(ictx->cct, 20) << "  diff " << diff << " end_exists=" << end_exists << dendl;
+	if (diff.empty())
+	  continue;
+
+	for (vector<ObjectExtent>::iterator q = p->second.begin(); q != p->second.end(); ++q) {
+	  ldout(ictx->cct, 20) << "diff_iterate object " << p->first
+			       << " extent " << q->offset << "~" << q->length
+			       << " from " << q->buffer_extents
+			       << dendl;
+	  uint64_t opos = q->offset;
+	  for (vector<pair<uint64_t,uint64_t> >::iterator r = q->buffer_extents.begin();
+	       r != q->buffer_extents.end();
+	       ++r) {
+	    interval_set<uint64_t> overlap;  // object extents
+	    overlap.insert(opos, r->second);
+	    overlap.intersection_of(diff);
+	    ldout(ictx->cct, 20) << " opos " << opos
+				 << " buf " << r->first << "~" << r->second
+				 << " overlap " << overlap
+				 << dendl;
+	    for (interval_set<uint64_t>::iterator s = overlap.begin();
+		 s != overlap.end();
+		 ++s) {
+	      uint64_t su_off = s.get_start() - opos;
+	      uint64_t logical_off = off + r->first + su_off;
+	      ldout(ictx->cct, 20) << "   overlap extent " << s.get_start() << "~" << s.get_len()
+				   << " logical "
+				   << logical_off << "~" << s.get_len()
+				   << dendl;
+	      cb(logical_off, s.get_len(), end_exists, arg);
+	    }
+	    opos += r->second;
+	  }
+	  assert(opos == q->offset + q->length);
+	}
+      }
+
+      left -= read_len;
+      off += read_len;
+    }
+
+    return 0;
+  }
+
   int simple_read_cb(uint64_t ofs, size_t len, const char *buf, void *arg)
   {
     char *dest_buf = (char *)arg;
@@ -2267,7 +2558,9 @@ reprotect_and_return_err:
 
   ssize_t read(ImageCtx *ictx, uint64_t ofs, size_t len, char *buf)
   {
-    return read_iterate(ictx, ofs, len, simple_read_cb, buf);
+    vector<pair<uint64_t,uint64_t> > extents;
+    extents.push_back(make_pair(ofs, len));
+    return read(ictx, extents, buf, NULL);
   }
 
   ssize_t read(ImageCtx *ictx, const vector<pair<uint64_t,uint64_t> >& image_extents, char *buf, bufferlist *pbl)
@@ -2291,7 +2584,6 @@ reprotect_and_return_err:
       cond.Wait(mylock);
     mylock.Unlock();
 
-    c->release();
     return ret;
   }
 
@@ -2326,7 +2618,6 @@ reprotect_and_return_err:
       cond.Wait(mylock);
     mylock.Unlock();
 
-    c->release();
     if (ret < 0)
       return ret;
 
@@ -2363,7 +2654,6 @@ reprotect_and_return_err:
       cond.Wait(mylock);
     mylock.Unlock();
 
-    c->release();
     if (ret < 0)
       return ret;
 
