@@ -51,6 +51,9 @@
 #include <memory>
 #include <tr1/memory>
 #include <errno.h>
+#include <boost/intrusive_ptr.hpp>
+// Because intusive_ptr clobbers our assert...
+#include "include/assert.h"
 
 
 #define CEPH_MON_PROTOCOL     10 /* cluster internal */
@@ -92,7 +95,6 @@ class AdminSocketHook;
 class MMonGetMap;
 class MMonGetVersion;
 class MMonSync;
-class MMonScrub;
 class MMonProbe;
 class MMonSubscribe;
 class MAuthRotating;
@@ -198,24 +200,6 @@ private:
 
   set<string> outside_quorum;
 
-  /**
-   * @defgroup scrub
-   * @{
-   */
-  version_t scrub_version;            ///< paxos version we are scrubbing
-  map<int,ScrubResult> scrub_result;  ///< results so far
-
-  /**
-   * trigger a cross-mon scrub
-   *
-   * Verify all mons are storing identical content
-   */
-  int scrub();
-  void handle_scrub(MMonScrub *m);
-  void _scrub(ScrubResult *r);
-  void scrub_finish();
-  void scrub_reset();
-
   /**
    * @defgroup Synchronization
    * @{
@@ -369,13 +353,17 @@ private:
 
     C_TrimEnable(Monitor *m) : mon(m) { }
     void finish(int r) {
-      mon->_trim_enable();
+      Mutex::Locker(mon->trim_lock);
+      // even if we are no longer the leader, we should re-enable trim if
+      // we have disabled it in the past. It doesn't mean we are going to
+      // do anything about it, but if we happen to become the leader
+      // sometime down the future, we sure want to have the trim enabled.
+      if (mon->trim_timeouts.empty())
+	mon->paxos->trim_enable();
+      mon->trim_enable_timer = NULL;
     }
   };
 
-  void _trim_enable();
-
-  void sync_obtain_latest_monmap(bufferlist &bl);
   void sync_store_init();
   void sync_store_cleanup();
   bool is_sync_on_going();
@@ -560,6 +548,7 @@ private:
 	version(0),
 	timeout(NULL),
 	sync_state(STATE_NONE),
+	crc(0),
 	crc_available(false),
 	attempts(0)
     { }
@@ -623,6 +612,9 @@ private:
 
       string prefix("paxos");
       paxos_synchronizer = mon->store->get_synchronizer(prefix);
+      version = mon->paxos->get_version();
+      generic_dout(10) << __func__ << " version " << version << dendl;
+
       synchronizer = mon->store->get_synchronizer(last_received_key,
 						  sync_targets);
       sync_update();
@@ -1257,8 +1249,8 @@ public:
   friend class PGMonitor;
   friend class LogMonitor;
 
-  QuorumService *health_monitor;
-  QuorumService *config_key_service;
+  boost::intrusive_ptr<QuorumService> health_monitor;
+  boost::intrusive_ptr<QuorumService> config_key_service;
 
   // -- sessions --
   MonSessionMap session_map;
@@ -1294,7 +1286,9 @@ public:
   void reply_command(MMonCommand *m, int rc, const string &rs, version_t version);
   void reply_command(MMonCommand *m, int rc, const string &rs, bufferlist& rdata, version_t version);
 
-
+  /**
+   * Handle Synchronization-related messages.
+   */
   void handle_probe(MMonProbe *m);
   /**
    * Handle a Probe Operation, replying with our name, quorum and known versions.
@@ -1421,7 +1415,6 @@ public:
   int preinit();
   int init();
   void init_paxos();
-  void refresh_from_paxos(bool *need_bootstrap);
   void shutdown();
   void tick();
 
@@ -1454,7 +1447,7 @@ private:
 public:
   class StoreConverter {
     const string path;
-    MonitorDBStore *db;
+    boost::scoped_ptr<MonitorDBStore> db;
     boost::scoped_ptr<MonitorStore> store;
 
     set<version_t> gvs;
@@ -1464,8 +1457,8 @@ public:
     version_t highest_accepted_pn;
 
    public:
-    StoreConverter(string path, MonitorDBStore *d)
-      : path(path), db(d), store(NULL),
+    StoreConverter(const string &path)
+      : path(path), db(NULL), store(NULL),
 	highest_last_pn(0), highest_accepted_pn(0)
     { }
 
@@ -1478,20 +1471,20 @@ public:
     int needs_conversion();
     int convert();
 
-    bool is_converting() {
-      return db->exists("mon_convert", "on_going");
-    }
-
    private:
 
     bool _check_gv_store();
 
     void _init() {
+      MonitorDBStore *db_ptr = new MonitorDBStore(path);
+      db.reset(db_ptr);
+
       MonitorStore *store_ptr = new MonitorStore(path);
       store.reset(store_ptr);
     }
 
     void _deinit() {
+      db.reset(NULL);
       store.reset(NULL);
     }
 
