@@ -38,7 +38,7 @@ class MonitorDBStore
   struct Op {
     uint8_t type;
     string prefix;
-    string key, endkey;
+    string key;
     bufferlist bl;
 
     Op() { }
@@ -46,27 +46,22 @@ class MonitorDBStore
       : type(t), prefix(p), key(k) { }
     Op(int t, const string& p, string k, bufferlist& b)
       : type(t), prefix(p), key(k), bl(b) { }
-    Op(int t, const string& p, string start, string end)
-      : type(t), prefix(p), key(start), endkey(end) { }
 
     void encode(bufferlist& encode_bl) const {
-      ENCODE_START(2, 1, encode_bl);
+      ENCODE_START(1, 1, encode_bl);
       ::encode(type, encode_bl);
       ::encode(prefix, encode_bl);
       ::encode(key, encode_bl);
       ::encode(bl, encode_bl);
-      ::encode(endkey, encode_bl);
       ENCODE_FINISH(encode_bl);
     }
 
     void decode(bufferlist::iterator& decode_bl) {
-      DECODE_START(2, decode_bl);
+      DECODE_START(1, decode_bl);
       ::decode(type, decode_bl);
       ::decode(prefix, decode_bl);
       ::decode(key, decode_bl);
       ::decode(bl, decode_bl);
-      if (struct_v >= 2)
-	::decode(endkey, decode_bl);
       DECODE_FINISH(decode_bl);
     }
   };
@@ -77,7 +72,7 @@ class MonitorDBStore
     enum {
       OP_PUT	= 1,
       OP_ERASE	= 2,
-      OP_COMPACT = 3,
+      OP_COMPACT_PREFIX = 3,
     };
 
     void put(string prefix, string key, bufferlist& bl) {
@@ -107,11 +102,7 @@ class MonitorDBStore
     }
 
     void compact_prefix(string prefix) {
-      ops.push_back(Op(OP_COMPACT, prefix, string()));
-    }
-
-    void compact_range(string prefix, string start, string end) {
-      ops.push_back(Op(OP_COMPACT, prefix, start, end));
+      ops.push_back(Op(OP_COMPACT_PREFIX, prefix, string()));
     }
 
     void encode(bufferlist& bl) const {
@@ -145,13 +136,13 @@ class MonitorDBStore
       return ops.size();
     }
 
-    void dump(ceph::Formatter *f) {
+    void dump(ceph::Formatter *f, bool dump_val=false) const {
       f->open_object_section("transaction");
       f->open_array_section("ops");
-      list<Op>::iterator it;
+      list<Op>::const_iterator it;
       int op_num = 0;
       for (it = ops.begin(); it != ops.end(); ++it) {
-	Op& op = *it;
+	const Op& op = *it;
 	f->open_object_section("op");
 	f->dump_int("op_num", op_num++);
 	switch (op.type) {
@@ -160,10 +151,12 @@ class MonitorDBStore
 	    f->dump_string("type", "PUT");
 	    f->dump_string("prefix", op.prefix);
 	    f->dump_string("key", op.key);
-	    ostringstream os;
-	    op.bl.hexdump(os);
 	    f->dump_unsigned("length", op.bl.length());
-	    f->dump_string("bl", os.str());
+	    if (dump_val) {
+	      ostringstream os;
+	      op.bl.hexdump(os);
+	      f->dump_string("bl", os.str());
+	    }
 	  }
 	  break;
 	case OP_ERASE:
@@ -173,12 +166,10 @@ class MonitorDBStore
 	    f->dump_string("key", op.key);
 	  }
 	  break;
-	case OP_COMPACT:
+	case OP_COMPACT_PREFIX:
 	  {
-	    f->dump_string("type", "COMPACT");
+	    f->dump_string("type", "COMPACT_PREFIX");
 	    f->dump_string("prefix", op.prefix);
-	    f->dump_string("start", op.key);
-	    f->dump_string("end", op.endkey);
 	  }
 	  break;
 	default:
@@ -195,7 +186,7 @@ class MonitorDBStore
     }
   };
 
-  int apply_transaction(MonitorDBStore::Transaction& t) {
+  int apply_transaction(const MonitorDBStore::Transaction& t) {
     KeyValueDB::Transaction dbt = db->get_transaction();
 
     if (do_dump) {
@@ -204,7 +195,7 @@ class MonitorDBStore
       bl.write_fd(dump_fd);
     }
 
-    list<pair<string, pair<string,string> > > compact;
+    list<string> compact_prefixes;
     for (list<Op>::const_iterator it = t.ops.begin(); it != t.ops.end(); ++it) {
       const Op& op = *it;
       switch (op.type) {
@@ -214,8 +205,8 @@ class MonitorDBStore
       case Transaction::OP_ERASE:
 	dbt->rmkey(op.prefix, op.key);
 	break;
-      case Transaction::OP_COMPACT:
-	compact.push_back(make_pair(op.prefix, make_pair(op.key, op.endkey)));
+      case Transaction::OP_COMPACT_PREFIX:
+	compact_prefixes.push_back(op.prefix);
 	break;
       default:
 	derr << __func__ << " unknown op type " << op.type << dendl;
@@ -225,13 +216,9 @@ class MonitorDBStore
     }
     int r = db->submit_transaction_sync(dbt);
     if (r >= 0) {
-      while (!compact.empty()) {
-	if (compact.front().second.first == string() &&
-	    compact.front().second.second == string())
-	  db->compact_prefix_async(compact.front().first);
-	else
-	  db->compact_range_async(compact.front().first, compact.front().second.first, compact.front().second.second);
-	compact.pop_front();
+      while (!compact_prefixes.empty()) {
+	db->compact_prefix(compact_prefixes.front());
+	compact_prefixes.pop_front();
       }
     }
     return r;
@@ -277,6 +264,7 @@ class MonitorDBStore
       return true;
     }
 
+    virtual void _get_chunk(Transaction &tx) = 0;
     virtual bool _is_valid() = 0;
 
   public:
@@ -291,14 +279,12 @@ class MonitorDBStore
     virtual bool has_next_chunk() {
       return !done && _is_valid();
     }
-    virtual void get_chunk_tx(Transaction &tx) = 0;
-    virtual void get_chunk(bufferlist& bl) {
+    virtual void get_chunk(bufferlist &bl) {
       Transaction tx;
-      get_chunk_tx(tx);
+      _get_chunk(tx);
       if (!tx.empty())
 	tx.encode(bl);
     }
-    virtual pair<string,string> get_next_key() = 0;
   };
   typedef std::tr1::shared_ptr<StoreIteratorImpl> Synchronizer;
 
@@ -325,7 +311,7 @@ class MonitorDBStore
      *			    differ from the one passed on to the function)
      * @param last_key[out] Last key in the chunk
      */
-    virtual void get_chunk_tx(Transaction &tx) {
+    virtual void _get_chunk(Transaction &tx) {
       assert(done == false);
       assert(iter->valid() == true);
 
@@ -343,15 +329,6 @@ class MonitorDBStore
       done = true;
     }
 
-    virtual pair<string,string> get_next_key() {
-      assert(iter->valid());
-      pair<string,string> r = iter->raw_key();
-      do {
-	iter->next();
-      } while (iter->valid() && sync_prefixes.count(iter->raw_key().first) == 0);
-      return r;
-    }
-
     virtual bool _is_valid() {
       return iter->valid();
     }
@@ -371,7 +348,7 @@ class MonitorDBStore
     virtual ~SinglePrefixStoreIteratorImpl() { }
 
   private:
-    virtual void get_chunk_tx(Transaction &tx) {
+    virtual void _get_chunk(Transaction &tx) {
       assert(done == false);
       assert(iter->valid() == true);
 
@@ -386,15 +363,6 @@ class MonitorDBStore
       done = true;
     }
 
-    virtual pair<string,string> get_next_key() {
-      // this method is only used by scrub on the whole store
-      // iterator.  also, the single prefix iterator has been dropped
-      // in later code.  we leave this here only for the benefit of
-      // backporting.
-      assert(0 == "this should not get called");
-      return make_pair(string(), string());
-    }
-
     virtual bool _is_valid() {
       return iter->valid();
     }
@@ -513,6 +481,14 @@ class MonitorDBStore
   }
 
   int open(ostream &out) {
+    db->options.write_buffer_size = g_conf->mon_leveldb_write_buffer_size;
+    db->options.cache_size = g_conf->mon_leveldb_cache_size;
+    db->options.block_size = g_conf->mon_leveldb_block_size;
+    db->options.bloom_size = g_conf->mon_leveldb_bloom_size;
+    db->options.compression_enabled = g_conf->mon_leveldb_compression;
+    db->options.max_open_files = g_conf->mon_leveldb_max_open_files;
+    db->options.paranoid_checks = g_conf->mon_leveldb_paranoid;
+    db->options.log_file = g_conf->mon_leveldb_log;
     return db->open(out);
   }
 
@@ -540,21 +516,13 @@ class MonitorDBStore
     os << path.substr(0, path.size() - pos) << "/store.db";
     string full_path = os.str();
 
-    LevelDBStore *db_ptr = new LevelDBStore(g_ceph_context, full_path);
+    LevelDBStore *db_ptr = new LevelDBStore(full_path);
     if (!db_ptr) {
       std::cout << __func__ << " error initializing level db back storage in "
 		<< full_path << std::endl;
       assert(0 != "MonitorDBStore: error initializing level db back storage");
     }
     db.reset(db_ptr);
-    db->options.write_buffer_size = g_conf->mon_leveldb_write_buffer_size;
-    db->options.cache_size = g_conf->mon_leveldb_cache_size;
-    db->options.block_size = g_conf->mon_leveldb_block_size;
-    db->options.bloom_size = g_conf->mon_leveldb_bloom_size;
-    db->options.compression_enabled = g_conf->mon_leveldb_compression;
-    db->options.max_open_files = g_conf->mon_leveldb_max_open_files;
-    db->options.paranoid_checks = g_conf->mon_leveldb_paranoid;
-    db->options.log_file = g_conf->mon_leveldb_log;
 
     if (g_conf->mon_debug_dump_transactions) {
       do_dump = true;
