@@ -29,7 +29,6 @@ class RGWHandler;
 void rgw_get_request_metadata(struct req_state *s, map<string, bufferlist>& attrs);
 int rgw_build_policies(RGWRados *store, struct req_state *s, bool only_bucket, bool prefetch_data);
 
-
 /**
  * Provide the base class for all ops.
  */
@@ -50,10 +49,13 @@ public:
   virtual int verify_params() { return 0; }
   virtual bool prefetch_data() { return false; }
   virtual int verify_permission() = 0;
+  virtual int verify_op_mask();
   virtual void execute() = 0;
   virtual void send_response() {}
   virtual void complete() { send_response(); }
   virtual const char *name() = 0;
+
+  virtual uint32_t op_mask() { return 0; }
 };
 
 class RGWGetObj : public RGWOp {
@@ -119,6 +121,7 @@ public:
   virtual int send_response_data(bufferlist& bl, off_t ofs, off_t len) = 0;
 
   virtual const char *name() { return "get_obj"; }
+  virtual uint32_t op_mask() { return RGW_OP_TYPE_READ; }
 };
 
 #define RGW_LIST_BUCKETS_LIMIT_MAX 10000
@@ -148,6 +151,7 @@ public:
   virtual bool should_get_stats() { return false; }
 
   virtual const char *name() { return "list_buckets"; }
+  virtual uint32_t op_mask() { return RGW_OP_TYPE_READ; }
 };
 
 class RGWStatAccount : public RGWOp {
@@ -172,6 +176,7 @@ public:
 
   virtual void send_response() = 0;
   virtual const char *name() { return "stat_account"; }
+  virtual uint32_t op_mask() { return RGW_OP_TYPE_READ; }
 };
 
 class RGWListBucket : public RGWOp {
@@ -203,6 +208,7 @@ public:
   virtual int get_params() = 0;
   virtual void send_response() = 0;
   virtual const char *name() { return "list_bucket"; }
+  virtual uint32_t op_mask() { return RGW_OP_TYPE_READ; }
 };
 
 class RGWGetBucketLogging : public RGWOp {
@@ -213,6 +219,7 @@ public:
 
   virtual void send_response() = 0;
   virtual const char *name() { return "get_bucket_logging"; }
+  virtual uint32_t op_mask() { return RGW_OP_TYPE_READ; }
 };
 
 class RGWStatBucket : public RGWOp {
@@ -229,6 +236,7 @@ public:
 
   virtual void send_response() = 0;
   virtual const char *name() { return "stat_bucket"; }
+  virtual uint32_t op_mask() { return RGW_OP_TYPE_READ; }
 };
 
 class RGWCreateBucket : public RGWOp {
@@ -248,6 +256,7 @@ public:
   virtual int get_params() { return 0; }
   virtual void send_response() = 0;
   virtual const char *name() { return "create_bucket"; }
+  virtual uint32_t op_mask() { return RGW_OP_TYPE_WRITE; }
 };
 
 class RGWDeleteBucket : public RGWOp {
@@ -262,6 +271,7 @@ public:
 
   virtual void send_response() = 0;
   virtual const char *name() { return "delete_bucket"; }
+  virtual uint32_t op_mask() { return RGW_OP_TYPE_DELETE; }
 };
 
 class RGWPutObjProcessor
@@ -330,6 +340,7 @@ public:
   virtual int get_data(bufferlist& bl) = 0;
   virtual void send_response() = 0;
   virtual const char *name() { return "put_obj"; }
+  virtual uint32_t op_mask() { return RGW_OP_TYPE_WRITE; }
 };
 
 class RGWPostObj : public RGWOp {
@@ -371,6 +382,7 @@ public:
   virtual int get_data(bufferlist& bl) = 0;
   virtual void send_response() = 0;
   virtual const char *name() { return "post_obj"; }
+  virtual uint32_t op_mask() { return RGW_OP_TYPE_WRITE; }
 };
 
 class RGWPutMetadata : public RGWOp {
@@ -398,6 +410,7 @@ public:
   virtual int get_params() = 0;
   virtual void send_response() = 0;
   virtual const char *name() { return "put_obj_metadata"; }
+  virtual uint32_t op_mask() { return RGW_OP_TYPE_WRITE; }
 };
 
 class RGWDeleteObj : public RGWOp {
@@ -412,6 +425,7 @@ public:
 
   virtual void send_response() = 0;
   virtual const char *name() { return "delete_obj"; }
+  virtual uint32_t op_mask() { return RGW_OP_TYPE_DELETE; }
 };
 
 class RGWCopyObj : public RGWOp {
@@ -473,6 +487,7 @@ public:
   virtual int get_params() = 0;
   virtual void send_response() = 0;
   virtual const char *name() { return "copy_obj"; }
+  virtual uint32_t op_mask() { return RGW_OP_TYPE_WRITE; }
 };
 
 class RGWGetACLs : public RGWOp {
@@ -488,6 +503,7 @@ public:
 
   virtual void send_response() = 0;
   virtual const char *name() { return "get_acls"; }
+  virtual uint32_t op_mask() { return RGW_OP_TYPE_READ; }
 };
 
 class RGWPutACLs : public RGWOp {
@@ -513,6 +529,7 @@ public:
   virtual int get_params() = 0;
   virtual void send_response() = 0;
   virtual const char *name() { return "put_acls"; }
+  virtual uint32_t op_mask() { return RGW_OP_TYPE_WRITE; }
 };
 
 class RGWGetCORS : public RGWOp {
@@ -528,6 +545,7 @@ public:
 
   virtual void send_response() = 0;
   virtual const char *name() { return "get_cors"; }
+  virtual uint32_t op_mask() { return RGW_OP_TYPE_READ; }
 };
 
 class RGWPutCORS : public RGWOp {
@@ -552,6 +570,7 @@ public:
   virtual int get_params() = 0;
   virtual void send_response() = 0;
   virtual const char *name() { return "put_cors"; }
+  virtual uint32_t op_mask() { return RGW_OP_TYPE_WRITE; }
 };
 
 class RGWDeleteCORS : public RGWOp {
@@ -566,6 +585,7 @@ public:
 
   virtual void send_response() = 0;
   virtual const char *name() { return "delete_cors"; }
+  virtual uint32_t op_mask() { return RGW_OP_TYPE_WRITE; }
 };
 
 class RGWOptionsCORS : public RGWOp {
@@ -585,6 +605,7 @@ public:
   void get_response_params(string& allowed_hdrs, string& exp_hdrs, unsigned *max_age);
   virtual void send_response() = 0;
   virtual const char *name() { return "options_cors"; }
+  virtual uint32_t op_mask() { return RGW_OP_TYPE_READ; }
 };
 
 class RGWInitMultipart : public RGWOp {
@@ -608,6 +629,7 @@ public:
   virtual int get_params() = 0;
   virtual void send_response() = 0;
   virtual const char *name() { return "init_multipart"; }
+  virtual uint32_t op_mask() { return RGW_OP_TYPE_WRITE; }
 };
 
 class RGWCompleteMultipart : public RGWOp {
@@ -636,6 +658,7 @@ public:
   virtual int get_params() = 0;
   virtual void send_response() = 0;
   virtual const char *name() { return "complete_multipart"; }
+  virtual uint32_t op_mask() { return RGW_OP_TYPE_WRITE; }
 };
 
 class RGWAbortMultipart : public RGWOp {
@@ -650,6 +673,7 @@ public:
 
   virtual void send_response() = 0;
   virtual const char *name() { return "abort_multipart"; }
+  virtual uint32_t op_mask() { return RGW_OP_TYPE_DELETE; }
 };
 
 class RGWListMultipart : public RGWOp {
@@ -678,6 +702,7 @@ public:
   virtual int get_params() = 0;
   virtual void send_response() = 0;
   virtual const char *name() { return "list_multipart"; }
+  virtual uint32_t op_mask() { return RGW_OP_TYPE_READ; }
 };
 
 #define MP_META_SUFFIX ".meta"
@@ -786,6 +811,7 @@ public:
   virtual int get_params() = 0;
   virtual void send_response() = 0;
   virtual const char *name() { return "list_bucket_multiparts"; }
+  virtual uint32_t op_mask() { return RGW_OP_TYPE_READ; }
 };
 
 class RGWDeleteMultiObj : public RGWOp {
@@ -818,6 +844,7 @@ public:
   virtual void send_partial_response(pair<string,int>& result) = 0;
   virtual void end_response() = 0;
   virtual const char *name() { return "multi_object_delete"; }
+  virtual uint32_t op_mask() { return RGW_OP_TYPE_DELETE; }
 };
 
 
