@@ -347,9 +347,9 @@ void dump_owner(struct req_state *s, string& id, string& name, const char *secti
 void dump_access_control(struct req_state *s, const char *origin, const char *meth,
                          const char *hdr, const char *exp_hdr, uint32_t max_age) {
   if (origin && (origin[0] != '\0')) {
-    s->cio->print("Access-Control-Allow-Origin: %s\n", origin?origin:"");
+    s->cio->print("Access-Control-Allow-Origin: %s\n", origin);
     if (meth && (meth[0] != '\0'))
-      s->cio->print("Access-Control-Allow-Methods: %s\n", meth?meth:"");
+      s->cio->print("Access-Control-Allow-Methods: %s\n", meth);
     if (hdr && (hdr[0] != '\0'))
       s->cio->print("Access-Control-Allow-Headers: %s\n", hdr);
     if (exp_hdr && (exp_hdr[0] != '\0')) {
@@ -1163,33 +1163,8 @@ void RGWRESTMgr::register_resource(string resource, RGWRESTMgr *mgr)
 {
   string r = "/";
   r.append(resource);
-
-  /* do we have a resource manager registered for this entry point? */
-  map<string, RGWRESTMgr *>::iterator iter = resource_mgrs.find(r);
-  if (iter != resource_mgrs.end()) {
-    delete iter->second;
-  }
   resource_mgrs[r] = mgr;
   resources_by_size.insert(pair<size_t, string>(r.size(), r));
-
-  /* now build default resource managers for the path (instead of nested entry points)
-   * e.g., if the entry point is /auth/v1.0/ then we'd want to create a default
-   * manager for /auth/
-   */
-
-  size_t pos = r.find('/', 1);
-
-  while (pos != r.size() - 1 && pos != string::npos) {
-    string s = r.substr(0, pos);
-
-    iter = resource_mgrs.find(s);
-    if (iter == resource_mgrs.end()) { /* only register it if one does not exist */
-      resource_mgrs[s] = new RGWRESTMgr; /* a default do-nothing manager */
-      resources_by_size.insert(pair<size_t, string>(s.size(), s));
-    }
-
-    pos = r.find('/', pos + 1);
-  }
 }
 
 void RGWRESTMgr::register_default_mgr(RGWRESTMgr *mgr)
@@ -1208,8 +1183,8 @@ RGWRESTMgr *RGWRESTMgr::get_resource_mgr(struct req_state *s, const string& uri)
   for (iter = resources_by_size.rbegin(); iter != resources_by_size.rend(); ++iter) {
     string& resource = iter->second;
     if (uri.compare(0, iter->first, resource) == 0 &&
-	(uri.size() == iter->first ||
-	 uri[iter->first] == '/')) {
+	(resource.size() == iter->first ||
+	 resource[iter->first] == '/')) {
       string suffix = uri.substr(iter->first);
       return resource_mgrs[resource]->get_resource_mgr(s, suffix);
     }
